<?php

/**
 * @file
 * Contains \Drupal\yamlform\Form\YamlFormUiElementAddForm.
 */

namespace Drupal\yamlform_ui\Form;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\yamlform\YamlFormInterface;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;

/**
 * Provides an add form for a YAML form element.
 */
class YamlFormUiElementAddForm extends YamlFormUiElementFormBase {

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, YamlFormInterface $yamlform = NULL, $type = NULL) {
    $this->yamlform = $yamlform;
    $parent_key = $this->getRequest()->get('parent');

    if ($parent_key) {
      $parent_element = $yamlform->getElementDecoded($parent_key);
      if (!$parent_element) {
        throw new NotFoundHttpException();
      }

      $form['#title'] = $this->t('@form_title: Add element to %element_title', [
        '@form_title' => $yamlform->label(),
        '%element_title' => (!empty($parent_element['#title'])) ? $parent_element['#title'] : $parent_key,
      ]);
    }
    else {
      $form['#title'] = $this->t('@form_title: Add element', [
        '@form_title' => $yamlform->label(),
        '%element_title' => (!empty($parent_element['#title'])) ? $parent_element['#title'] : $parent_key,
      ]);
    }

    if (empty($type)) {
      $form['type'] = [
        '#type' => 'select',
        '#title' => t('Add a new element'),
        '#options' => $this->getElementOptions($parent_key),
        '#required' => TRUE,
        '#empty_option' => $this->t('- Select an element type -'),
      ];

      $form['actions'] = ['#type' => 'actions'];
      $form['actions']['submit'] = [
        '#type' => 'submit',
        '#value' => $this->t('Continue'),
        '#button_type' => 'primary',
        '#validate' => ['::skipValidate'],
        '#submit' => ['::next'],
      ];
      return $form;
    }
    else {
      $this->element['#type'] = $type;
      return parent::buildForm($form, $form_state, $yamlform, NULL, $parent_key);
    }
  }
  /**
   * Skip form validation handler for the 'next' action.
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   */
  public function skipValidate(array &$form, FormStateInterface $form_state) {
    // Do nothing.
  }

  /**
   * Form submission handler for the 'next' action.
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   */
  public function next(array &$form, FormStateInterface $form_state) {
    $route_parameters = [
      'yamlform' => $this->yamlform->id(),
      'type' => $form_state->getValue('type'),
    ];
    $route_options = [];
    if ($parent_key = $this->getRequest()->get('parent')) {
      $route_options['query'] = ['parent' => $this->getRequest()->get('parent')];
    }
    $redirect_url = Url::fromRoute('entity.yamlform_ui.element.add_form', $route_parameters, $route_options);
    $form_state->setRedirectUrl($redirect_url);
  }

  /**
   * Get available element as options.
   *
   * @param string $parent_key
   *   The parent the element is being added to.
   *
   * @return array
   *   An associative array containing available element as options.
   */
  protected function getElementOptions($parent_key) {
    /** @var \Drupal\yamlform\YamlFormElementManagerInterface $yamlform_element_manager */
    $yamlform_element_manager = \Drupal::service('plugin.manager.yamlform.element');

    /** @var \Drupal\Core\Render\ElementInfoManagerInterface $element_manager */
    $element_manager = \Drupal::service('plugin.manager.element_info');

    $definitions = $yamlform_element_manager->getDefinitions();
    $options = [];
    foreach ($definitions as $plugin_id => $definition) {
      // Skip hidden elements and elements that are missing definitions.
      if (!empty($definition['hidden']) || !$element_manager->hasDefinition($plugin_id)) {
        continue;
      }

      /** @var \Drupal\yamlform\YamlFormElementInterface $yamlform_element */
      $yamlform_element = $yamlform_element_manager->createInstance($plugin_id);

      // If parent_key is set, skip root only elements.
      if ($parent_key  && $yamlform_element->isRoot([])) {
        continue;
      }

      $options[(string) $yamlform_element->getPluginGroup()][$plugin_id] = (string) $yamlform_element->getPluginLabel();
    }

    // Sort each optgroup by element labels.
    foreach ($options as &$optgroup) {
      if (is_array($optgroup)) {
        asort($optgroup);
      }
    }
    ksort($options);

    // Make sure 'Basic' options are always first.
    $basic_optgroup = (string) $this->t('Basic elements');
    $basic_options = $options[$basic_optgroup];
    unset($options[$basic_optgroup]);
    $options = [$basic_optgroup => $basic_options] + $options;
    return $options;
  }

}
