<?php

/**
 * @file
 * Definition of Drupal\yamlform\Tests\YamlFormEntityElementsValidationUnitTest.
 */

namespace Drupal\yamlform\Tests;

use Drupal\Core\Url;
use Drupal\yamlform\YamlFormEntityElementsValidator;
use Drupal\KernelTests\KernelTestBase;

/**
 * Tests YAML form entity elements validation.
 *
 * @group YamlForm
 */
class YamlFormEntityElementsValidationUnitTest extends KernelTestBase {

  /**
   * Modules to enable.
   *
   * @var array
   */
  public static $modules = ['system', 'yamlform', 'user'];

  /**
   * The YAML form elements validator.
   *
   * @var \Drupal\yamlform\YamlFormEntityElementsValidator
   */
  protected $validator;

  /**
   * {@inheritdoc}
   */
  protected function setUp() {
    parent::setUp();
    $this->validator = new YamlFormEntityElementsValidator();
  }

  /**
   * Tests validating elements.
   */
  public function testValidate() {
    $tests = [
      /*
      [
        '', // Elements.
        '', // Original elements.
        [], // Validation error message.
      ],
      */

      // Check required.
      [
        '',
        '',
        ['Elements are required'],
      ],

      // Check invalid YAML.
      [
        "not\nvalid\nyaml",
        '',
        ['Elements are not valid. Unable to parse at line 1 (near &quot;not&quot;).'],
      ],

      // Check elements are an array.
      [
        'string',
        '',
        ['Elements are not valid. YAML must contain an associative array of elements.'],
      ],

      // Check duplicate names.
      [
        "name:
  '#type': textfield
duplicate:
  name:
    '#type': textfield",
        '',
        ['Elements contain a duplicate element name <em class="placeholder">name</em> found on lines 1 and 4.'],
      ],

      // Check duplicate name with single and double quotes.
      [
        "'name' :
  '#type': textfield
duplicate:
  \"name\":
    '#type': textfield",
        '',
        ['Elements contain a duplicate element name <em class="placeholder">name</em> found on lines 1 and 4.'],
      ],

      // Check ignored properties.
      [
        "'tree':
  '#tree': true
  '#submit' : 'function_name'",
        '',
        [
          'Elements contain a unsupported <em class="placeholder">#tree</em> property found on line 2.',
          'Elements contain a unsupported <em class="placeholder">#submit</em> property found on line 3.',
        ],
      ],

      // Check validate submissions.
      [
        "name_changed:
  '#type': 'textfield'",
        "name:
  '#type': 'textfield'",
        [
          'The <em class="placeholder">name</em> element can not be removed because the <em class="placeholder">Test</em> form has <a href="http://example.com">results</a>.<ul><li><a href="http://example.com">Delete all submissions</a> to this form.</li><li><a href="/admin/modules">Enable the YAML form UI module</a> and safely delete this element.</li><li>Hide this element by setting its <code>\'#access\'</code> property to <code>false</code>.</li></ul>',
        ],
      ],

      // Check validate rendering.
      [
        "machine_name:
  '#type': 'machine_name'
  '#machine_name':
     source:
      broken",
        "machine_name:
  '#type': 'machine_name'
  '#machine_name':
     source:
      broken",
        [
          'Unable to render elements, please view the below message and the error log.<ul><li>Query condition &#039;yamlform_submission.yamlform_id IN ()&#039; cannot be empty.</li></ul>',
        ],
      ],
    ];

    foreach ($tests as $test) {
      $yamlform = $this->getMock('\Drupal\yamlform\YamlFormInterface');
      $yamlform->expects($this->any())
        ->method('getElementsRaw')
        ->will($this->returnValue($test[0]));
      $yamlform->expects($this->any())
        ->method('getElementsOriginalRaw')
        ->will($this->returnValue($test[1]));
      $yamlform->expects($this->any())
        ->method('hasSubmissions')
        ->will($this->returnValue(TRUE));
      $yamlform->expects($this->any())
        ->method('label')
        ->will($this->returnValue('Test'));
      $yamlform->expects($this->any())
        ->method('toUrl')
        ->will($this->returnValue(Url::fromUri('http://example.com')));

      $messages = $this->validator->validate($yamlform);
      foreach ($messages as $index => $message) {
        $messages[$index] = (string) $message;
      }
      $this->assertEquals($messages, $test[2]);
    }
  }

}
